package org.ow2.util.ee.metadata.ejbjar.impl;


import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.ejb.ApplicationException;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.deploy.impl.deployable.metadata.DeployableMetadata;
import org.ow2.util.ee.metadata.common.api.struct.IJInterceptors;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarClassMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarDeployableMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.InterceptorType;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJCommonBean;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JApplicationException;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.EJB3;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * This class represents the metadata of all classes of an EjbJar file. From
 * this class, we can get metadata of all beans.
 * @param <E> ejb jar type
 * @author Florent Benoit
 */
public class EjbJarDeployableMetadata<E extends EJBDeployable<E>> extends DeployableMetadata<E> implements
        IEjbJarDeployableMetadata<E> {

    /**
     * Constructor.
     * @param deployable the ejb-jar deployable
     */
    public EjbJarDeployableMetadata(final E deployable) {
        super(deployable);
        classesAnnotationMetadata = new HashMap<String, IEjbJarClassMetadata<E>>();
    }

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(EjbJarDeployableMetadata.class);

    /**
     * List of class annotations metadata.
     */
    private Map<String, IEjbJarClassMetadata<E>> classesAnnotationMetadata = null;

    /**
     * Link to the Deployment Descriptor object.
     */
    private EJB3 ejb3 = null;

    /**
     * List of application exceptions used on this ejb-jar.
     */
    private Map<String, ApplicationException> applicationExceptions = null;

    /**
     * List of default interceptors by type (business or lifecycle).
     */
    private Map<InterceptorType, List<JClassInterceptor>> defaultInterceptors = null;

    /**
     * List of default interceptors classes.
     */
    private IJInterceptors defaultInterceptorsClasses = null;

    /**
     * Add annotation metadata for a given class.
     * @param classAnnotationMetadata annotation metadata of a class.
     */
    public void addEjbJarClassMetadata(final IEjbJarClassMetadata<E> classAnnotationMetadata) {
        String key = classAnnotationMetadata.getClassName();
        // already exists ?
        if (classesAnnotationMetadata.containsKey(key)) {
            String msg = logger.getI18n().getMessage("EjbJarAnnotationMetadata.addClassAnnotationMetadata.alreadyPresent", key);
            logger.debug(msg);
            throw new IllegalStateException(msg);
        }
        classesAnnotationMetadata.put(key, classAnnotationMetadata);
    }

    /**
     * Gets the class metadata for the given ejb-name.
     * @return class metadata or null if not found
     * @param ejbName the name of the EJB.
     */
    public IEjbJarClassMetadata<E> getEjbJarClassMetadataForEjbName(final String ejbName) {
        for (IEjbJarClassMetadata<E> metadata : classesAnnotationMetadata.values()) {
            IJCommonBean bean = metadata.getJCommonBean();
            if (bean != null && ejbName.equals(bean.getName())) {
                return metadata;
            }
        }
        return null;
    }

    /**
     * Get class annotation metadata.
     * @param className key of the map of annotations bean.
     * @return Bean annotation metadata of a given name or null if class not found.
     */
    public IEjbJarClassMetadata<E> getEjbJarClassMetadata(final String className) {
        return classesAnnotationMetadata.get(className);
    }

    /**
     * Get collections of bean annotation metadata.
     * @return collections of bean annotation metadata.
     */
    public Collection<IEjbJarClassMetadata<E>> getEjbJarClassMetadataCollection() {
        return classesAnnotationMetadata.values();
    }

    /**
     * @return the ejb3 deployment descriptor object.
     */
    public EJB3 getEjb3() {
        return ejb3;
    }

    /**
     * Sets the ejb3 deployment descriptor object.
     * @param ejb3 the ejb3 deployment descriptor object.
     */
    public void setEjb3(final EJB3 ejb3) {
        this.ejb3 = ejb3;
    }

    /**
     * Gets the list of application exceptions defined on this ejb jar metadata.
     * @return the list of application exceptions defined on this ejb jar
     *         metadata.
     */
    public Map<String, ApplicationException> getApplicationExceptions() {
        if (applicationExceptions != null) {
            return applicationExceptions;
        }

        // compute it
        applicationExceptions = new HashMap<String, ApplicationException>();

        // For each class, look if it is an application exception
        for (IEjbJarClassMetadata<E> classMetadata : getEjbJarClassMetadataCollection()) {
            ApplicationException appException = classMetadata.getApplicationException();
            // found it then add it in the map.
            if (appException != null) {
                applicationExceptions.put(classMetadata.getClassName().replaceAll("/", "."), appException);
            }
        }

        // Add a default application exception (for checked exception)
        applicationExceptions.put("DEFAULT", new JApplicationException());

        return applicationExceptions;
    }


    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt; (interceptor classes)
     * of default interceptors that enhancer will use.
     */
    public Map<InterceptorType, List<JClassInterceptor>> getDefaultInterceptors() {
        return defaultInterceptors;
    }

    /**
     * Sets the list of default interceptors that enhancers will use.<br>
     * These interceptors are defined by XML DD.
     * @param defaultInterceptors list of interceptors that enhancer will use.
     */
    public void setDefaultInterceptors(final Map<InterceptorType, List<JClassInterceptor>> defaultInterceptors) {
        this.defaultInterceptors = defaultInterceptors;
    }

    /**
     * @return object representing list of default interceptor classes.
     */
    public IJInterceptors getDefaultInterceptorsClasses() {
        return defaultInterceptorsClasses;
    }

    /**
     * Sets the object representing the default interceptor classes.
     * @param defaultInterceptorsClasses list of classes
     */
    public void setDefaultInterceptorsClasses(final IJInterceptors defaultInterceptorsClasses) {
        this.defaultInterceptorsClasses = defaultInterceptorsClasses;
    }

}
