/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: EjbJarDeployableMetadataFactory.java 3600 2008-07-04 15:25:28Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl;

import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.ow2.util.archive.api.ArchiveException;
import org.ow2.util.archive.api.IArchive;
import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.deploy.api.deployable.metadata.DeployableMetadataException;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarDeployableMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarDeployableMetadataFactory;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.EjbJarDeployableMetadataConfigurator;
import org.ow2.util.ee.metadata.ejbjar.impl.specific.ISpecificEjbJarDeployableFactory;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.EJB3DeploymentDesc;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.EJB3DeploymentDescException;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.helper.ExtractUsefullClasses;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.helper.MetadataMerge;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.EJB3;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.scan.api.IClassesLocator;
import org.ow2.util.scan.api.IScanner;
import org.ow2.util.scan.api.ScanException;
import org.ow2.util.scan.api.classlocator.ArchiveClassesLocator;
import org.ow2.util.scan.api.classlocator.ClassLoaderClassesLocator;
import org.ow2.util.scan.api.configurator.IArchiveConfigurator;
import org.ow2.util.scan.impl.ASMScannerImpl;

/**
 * Factory to contruct ejb jar deployable metadata.
 * @param <E> ejb-jar type (EJB3, EJB2.1)
 * @author Gael Lalire
 */
public class EjbJarDeployableMetadataFactory<E extends EJBDeployable<E>> implements
        IEjbJarDeployableMetadataFactory<E> {

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(EjbJarDeployableMetadataFactory.class);

    /**
     * Logger.
     */
    private static final String CLASS_SUFFIX = ".class";


    /**
     * Class scanner.
     */
    private IScanner scanner;

    /**
     * Constructor.
     * @throws ScanException if no archive scanner factory can be found
     */
    public EjbJarDeployableMetadataFactory() throws ScanException {
        scanner = new ASMScannerImpl();
    }

    /**
     * Constructor a scanner.
     * @param scanner the scanner
     */
    public EjbJarDeployableMetadataFactory(final IScanner scanner) {
        this.scanner = scanner;
    }

    /**
     * @param ejb3Deployable an ejb3 deployable
     * @return metadata constructed using ejb3Deployable
     * @throws DeployableMetadataException if metadata can not be created
     */
    public IEjbJarDeployableMetadata<E> createDeployableMetadata(final E ejb3Deployable) throws DeployableMetadataException {
        return createDeployableMetadata(ejb3Deployable, null, null);
    }

    /**
     * @param deployable an ejb3 deployable
     * @param classLoader a classloader to use if a class is not in archive
     * @return metadata constructed using ejb3Deployable
     * @throws DeployableMetadataException if metadata can not be created
     */
    public IEjbJarDeployableMetadata<E> createDeployableMetadata(final E deployable, final ClassLoader classLoader)
            throws DeployableMetadataException {
        List<? extends IClassesLocator> classesLocatorList = null;
        if (classLoader != null) {
            classesLocatorList = Arrays.asList(new ClassLoaderClassesLocator(classLoader));
        }
        return createDeployableMetadata(deployable, classesLocatorList, null);
    }

    /**
     * Extract a class list from xml.
     * @param ejb3 representation of ejb-jar.xml
     * @param classNames a set to fill with classes name referenced in ejb-jar.xml
     */
    private void completeClassList(final EJB3 ejb3, final Set<String> classNames) {
        for (String className : ExtractUsefullClasses.extract(ejb3)) {
            classNames.add(className.replace('.', '/'));
        }
    }


    /**
     * @param ejb3Deployable an ejb3 deployable
     * @param ejbJarExternLibraries extern libraries
     * @param specificDeployableFactoryList specific factories
     * @return metadata constructed using ejb3Deployable
     * @throws DeployableMetadataException if metadata can not be created
     */
    public IEjbJarDeployableMetadata<E> createDeployableMetadata(final E ejb3Deployable,
            final List<? extends IClassesLocator> ejbJarExternLibraries,
            final List<? extends ISpecificEjbJarDeployableFactory<E>> specificDeployableFactoryList)
            throws DeployableMetadataException {

        // for time debugging
        long tAnalyzeStart = System.currentTimeMillis();

        IArchive archive = ejb3Deployable.getArchive();

        // Get EJB3 DD.
        EJB3 ejb3;
        try {
            ejb3 = EJB3DeploymentDesc.getEjb3(archive);
        } catch (EJB3DeploymentDescException e) {
            throw new DeployableMetadataException(e);
        }

        EjbJarDeployableMetadataConfigurator<E> ejbJarArchiveMetadataConfigurator = new EjbJarDeployableMetadataConfigurator<E>(
                ejb3Deployable);
        Set<String> classNameSet = new HashSet<String>();

        if (specificDeployableFactoryList != null) {
            for (ISpecificEjbJarDeployableFactory<E> specificEjbJarDeployableFactory : specificDeployableFactoryList) {
                ejbJarArchiveMetadataConfigurator.addSpecificMetadataConfigurator(specificEjbJarDeployableFactory
                        .beforeScan(ejb3Deployable, classNameSet));
            }
        }

        // if metadata is complete then filter class to scan
        if (ejb3 != null && ejb3.isMetadataComplete()) {
            // disable annotation parsing
            ejbJarArchiveMetadataConfigurator.setAnnotationParsingDesactived(true);

        } else {
            // enable annotation parsing because metadata descriptor is not
            // complete
            ejbJarArchiveMetadataConfigurator.setAnnotationParsingDesactived(false);

            Iterator<String> entryIterator;
            try {
                entryIterator = archive.getEntries();
            } catch (ArchiveException e) {
                throw new DeployableMetadataException(e);
            }
            while (entryIterator.hasNext()) {
                String entry = entryIterator.next();
                if (entry.endsWith(CLASS_SUFFIX)) {
                    classNameSet.add(entry.substring(0, entry.length() - CLASS_SUFFIX.length()));
                }
            }
        }

        // get class to scan from deployment descriptor
        completeClassList(ejb3, classNameSet);

        // singleton
        List<IArchiveConfigurator> archiveConfiguratorList = Arrays
                .asList((IArchiveConfigurator) ejbJarArchiveMetadataConfigurator);

        IClassesLocator ejb3ClassesLocator = new ArchiveClassesLocator(archive);

        // begin scan
        List<String> scannedClasses = new ArrayList<String>();
        List<String> nextClassNameToScan = new ArrayList<String>();
        nextClassNameToScan.addAll(classNameSet);

        try {
            while (nextClassNameToScan.size() != 0) {
                // recalculate classNameSet
                classNameSet.clear();
                for (String className : nextClassNameToScan) {
                    URL url = ejb3ClassesLocator.getClassURL(className);
                    if (url == null && ejbJarExternLibraries != null) {
                        for (IClassesLocator classesLocator : ejbJarExternLibraries) {
                            url = classesLocator.getClassURL(className);
                            if (url != null) {
                                break;
                            }
                        }
                    }
                    if (url != null) {
                        scanner.scanClass(url, classNameSet, archiveConfiguratorList);
                    }
                    // we add to scanned even if there was no scan (class not
                    // found)
                    scannedClasses.add(className);
                }

                // keep what is not scanned
                nextClassNameToScan.clear();
                for (String className : classNameSet) {
                    if (!scannedClasses.contains(className)) {
                        nextClassNameToScan.add(className);
                    }
                }

            }
        } catch (ScanException e) {
            throw new DeployableMetadataException("Error during scan", e);
        }

        // scan finished

        EjbJarDeployableMetadata<E> ejbJarArchiveMetadata = ejbJarArchiveMetadataConfigurator.getEjbJarArchiveMetadata();

        // prepare the metadata for merge
        ejbJarArchiveMetadata.setEjb3(ejb3);

        // Merge DD with annotations metadata
        MetadataMerge.merge(ejbJarArchiveMetadata);


        if (specificDeployableFactoryList != null) {
            for (ISpecificEjbJarDeployableFactory<E> specificEjbJarDeployableFactory : specificDeployableFactoryList) {
                specificEjbJarDeployableFactory.afterScan();
            }
        }

        // time if debugging
        if (LOGGER.isDebugEnabled()) {
            long tAnalyzeStartEnd = System.currentTimeMillis();
            if (LOGGER.isDebugEnabled()) {
                LOGGER.debug("Analyze of file {0} took {1} ms.", archive.getName(), (tAnalyzeStartEnd - tAnalyzeStart));
            }
        }

        return ejbJarArchiveMetadata;
    }

    /**
     * @return the class returned by creation of metadata :
     *         IEJB3DeployableMetadata.class
     */
    @SuppressWarnings("unchecked")
    public Class<IEjbJarDeployableMetadata<E>> getDeployableMetadataClass() {
        return (Class<IEjbJarDeployableMetadata<E>>) (Class<?>) IEjbJarDeployableMetadata.class;
    }

}
