/**
 * EasyBeans
 * Copyright (C) 2006-2007 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EjbJarMethodMetadata.java 3340 2008-05-29 10:12:57Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl;

import static org.ow2.util.ee.metadata.common.impl.CommonMetadata.concatStringBuilder;

import java.util.List;
import java.util.Map;

import javax.ejb.Remove;
import javax.ejb.TransactionAttributeType;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.common.api.interfaces.IAnnotationSecurityPermitAll;
import org.ow2.util.ee.metadata.common.api.interfaces.IAnnotationSecurityRolesAllowed;
import org.ow2.util.ee.metadata.common.api.interfaces.IEJBInterceptors;
import org.ow2.util.ee.metadata.common.api.interfaces.IInterceptorExcludeDefaultInterceptors;
import org.ow2.util.ee.metadata.common.api.interfaces.ITransactionAttribute;
import org.ow2.util.ee.metadata.common.api.struct.IJInterceptors;
import org.ow2.util.ee.metadata.common.impl.CommonMethodMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarClassMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarFieldMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarMethodMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IJClassInterceptor;
import org.ow2.util.ee.metadata.ejbjar.api.InterceptorType;
import org.ow2.util.ee.metadata.ejbjar.api.TransactionAttributeLevel;
import org.ow2.util.scan.api.metadata.structures.JMethod;

/**
 * This class represents the annotation metadata of a method.
 * @param <E> deployable type
 * @author Florent Benoit
 */
public class EjbJarMethodMetadata<E extends EJBDeployable<E>> extends
        CommonMethodMetadata<IEjbJarClassMetadata<E>, IEjbJarMethodMetadata<E>, IEjbJarFieldMetadata<E>> implements Cloneable,
        ITransactionAttribute, IEJBInterceptors, IAnnotationSecurityRolesAllowed, IAnnotationSecurityPermitAll,
        IInterceptorExcludeDefaultInterceptors, IEjbJarMethodMetadata<E> {

    /**
     * Logger.
     */
    //private static Log logger = LogFactory.getLog(MethodAnnotationMetadata.class);

    /**
     * Type of transaction.
     */
    private TransactionAttributeType transactionAttributeType = null;

    /**
     * Level of transaction (when overriding with XML, need to know if the level is sufficient or not).
     */
    private TransactionAttributeLevel transactionAttributeLevel = TransactionAttributeLevel.ANNOTATION;

    /**
     * &#64;{@link javax.ejb.Remove} annotation.
     */
    private Remove remove = null;

    /**
     * This method is a business method ?
     */
    private boolean businessMethod = false;

    /**
     * PostConstruct method ?
     */
    private boolean postConstruct = false;

    /**
     * PreDestroy method ?
     */
    private boolean preDestroy = false;

    /**
     * PostActivate method ?
     */
    private boolean postActivate = false;

    /**
     * PrePassivate method ?
     */
    private boolean prePassivate = false;

    /**
     * &#64;{@link javax.interceptor.AroundInvoke} method used by interceptors ?
     */
    private boolean aroundInvoke = false;

    /**
     * &#64;{@link javax.interceptor.ExcludeClassInterceptors} method ?
     */
    private boolean excludeClassInterceptors = false;

    /**
     * &#64;{@link javax.ejb.Timeout} method ?
     */
    private boolean timeout = false;

    /**
     * EasyBeans global interceptors.<br>
     * These interceptors correspond to a list of interceptors
     * that need to be present first on this current method.
     */
    private List<? extends IJClassInterceptor> globalEasyBeansInterceptors = null;

    /**
     * EasyBeans method interceptors. These interceptors correspond to a list of Interceptors like security or transaction.
     */
    private List<? extends IJClassInterceptor> interceptors = null;

    /**
     * List of annotation interceptors.
     */
    private IJInterceptors annotationInterceptors = null;

    /**
     * User interceptors. These interceptors correspond to a list of Interceptor that user has specified in its bean class.
     * Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt;
     */
    private Map<InterceptorType, List<? extends IJClassInterceptor>> userInterceptors = null;

    /**
     * List of roles that are allowed on this class/bean.
     */
    private List<String> rolesAllowed = null;

    /**
     * This class/bean has the PermitAll annotation.
     */
    private boolean permitAll = false;

    /**
     * This class/bean has the DenyAll annotation.
     */
    private boolean denyAll = false;

    /**
     * &#64;{@link javax.interceptor.ExcludeDefaultInterceptors} method ?
     */
    private boolean excludeDefaultInterceptors = false;

    /**
     * Constructor.
     * @param jMethod the method on which we will set/add metadata
     * @param classAnnotationMetadata the parent metadata.
     */
    public EjbJarMethodMetadata(final JMethod jMethod, final IEjbJarClassMetadata<E> classAnnotationMetadata) {
        super(jMethod, classAnnotationMetadata);
    }

    /**
     * @return name of the method
     */
    public String getMethodName() {
        return getJMethod().getName();
    }

    /**
     * @return transaction Attribute type.
     * @see javax.ejb.TransactionAttributeType
     */
    public TransactionAttributeType getTransactionAttributeType() {
        return this.transactionAttributeType;
    }

    /**
     * Sets Transaction Attribute Level.
     * @param transactionAttributeLevel the level of transaction.
     */
    public void setTransactionAttributeLevel(final TransactionAttributeLevel transactionAttributeLevel) {
        this.transactionAttributeLevel = transactionAttributeLevel;
    }

    /**
     * @return transaction Attribute Level.
     */
    public TransactionAttributeLevel getTransactionAttributeLevel() {
        return this.transactionAttributeLevel;
    }

    /**
     * Sets Transaction Attribute Type.
     * @see javax.ejb.TransactionAttributeType
     * @param transactionAttributeType the type of transaction.
     */
    public void setTransactionAttributeType(final TransactionAttributeType transactionAttributeType) {
        this.transactionAttributeType = transactionAttributeType;
    }

    /**
     * @return true if the method is a business method.
     */
    public boolean isBusinessMethod() {
        return this.businessMethod;
    }

    /**
     * This method is a business method.
     * @param flag true/false if method is a business method.
     */
    public void setBusinessMethod(final boolean flag) {
        this.businessMethod = flag;
    }

    /**
     * @return true if the method is a lifecycle method.
     */
    public boolean isLifeCycleMethod() {
        return  isPostActivate() || isPostConstruct() || isPreDestroy() || isPrePassivate();
    }

    /**
     * @return remove attributes  &#64;{@link javax.ejb.Remove} attributes
     */
    public Remove getJRemove() {
        return this.remove;
    }

    /**
     * Sets &#64;{@link javax.ejb.Remove} attribute.
     * @param remove contains the attribute with retainIfException.
     */
    public void setRemove(final Remove remove) {
        this.remove = remove;
    }



    /**
     * @return string representation
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        String titleIndent = " ";

        // classname
        sb.append(titleIndent);
        sb.append(this.getClass().getName().substring(this.getClass().getPackage().getName().length() + 1));
        sb.append("[\n");

        // Add super class toString()
        sb.append(super.toString());

        // Method
        concatStringBuilder("jMethod", getJMethod(), sb);

        // transactionAttributeType
        concatStringBuilder("transactionAttributeType", this.transactionAttributeType, sb);

        // transactionAttributeLevel
        concatStringBuilder("transactionAttributeLevel", this.transactionAttributeLevel, sb);

        // remove
        concatStringBuilder("remove", this.remove, sb);

        // businessMethod
        concatStringBuilder("businessMethod", Boolean.valueOf(this.businessMethod), sb);

        // aroundInvoke
        concatStringBuilder("aroundInvoke", Boolean.valueOf(this.aroundInvoke), sb);

        // postConstruct
        concatStringBuilder("postConstruct", Boolean.valueOf(this.postConstruct), sb);

        // preDestroy
        concatStringBuilder("preDestroy", Boolean.valueOf(this.preDestroy), sb);

        // postActivate
        concatStringBuilder("postActivate", Boolean.valueOf(this.postActivate), sb);

        // prePassivate
        concatStringBuilder("prePassivate", Boolean.valueOf(this.prePassivate), sb);

        // timeout
        concatStringBuilder("timeout", Boolean.valueOf(this.timeout), sb);

        // annotation Interceptors
        concatStringBuilder("annotationInterceptors", this.annotationInterceptors, sb);

        // interceptors
        concatStringBuilder("interceptors", this.interceptors, sb);

        // rolesAllowed
        concatStringBuilder("rolesAllowed", this.rolesAllowed, sb);

        // permitAll
        concatStringBuilder("permitAll", Boolean.valueOf(this.permitAll), sb);

        // denyAll
        concatStringBuilder("denyAll", Boolean.valueOf(this.denyAll), sb);

        // denyAll
        concatStringBuilder("excludeDefaultInterceptors", Boolean.valueOf(this.excludeDefaultInterceptors), sb);

        sb.append(titleIndent);
        sb.append("]\n");
        return sb.toString();
    }

    /**
     * @return true if method has  &#64;{@link javax.ejb.PostActivate}
     */
    public boolean isPostActivate() {
        return this.postActivate;
    }

    /**
     * Sets true if method has &#64;{@link javax.ejb.PostActivate}.
     * @param postActivate true/false.
     */
    public void setPostActivate(final boolean postActivate) {
        this.postActivate = postActivate;
    }

    /**
     * @return true if method has &#64;{@link javax.annotation.PostConstruct}.
     */
    public boolean isPostConstruct() {
        return this.postConstruct;
    }

    /**
     * Sets true if method has &#64;{@link javax.annotation.PostConstruct}.
     * @param postConstruct true/false.
     */
    public void setPostConstruct(final boolean postConstruct) {
        this.postConstruct = postConstruct;
    }

    /**
     * @return true if method has &#64;{@link javax.annotation.PreDestroy}.
     */

    public boolean isPreDestroy() {
        return this.preDestroy;
    }

    /**
     * Sets true if method has &#64;{@link javax.annotation.PreDestroy}.
     * @param preDestroy true/false.
     */
    public void setPreDestroy(final boolean preDestroy) {
        this.preDestroy = preDestroy;
    }

    /**
     * @return true if method has &#64;{@link javax.ejb.PrePassivate}.
     */

    public boolean isPrePassivate() {
        return this.prePassivate;
    }

    /**
     * Sets true if method has &#64;{@link javax.ejb.PrePassivate}.
     * @param prePassivate true/false.
     */
    public void setPrePassivate(final boolean prePassivate) {
        this.prePassivate = prePassivate;
    }

    /**
     * @return true if method has &#64;{@link javax.ejb.Timeout}.
     */
    public boolean isTimeout() {
        return this.timeout;
    }

    /**
     * Sets true if method has &#64;{@link javax.ejb.Timeout}.
     * @param timeout true/false.
     */
    public void setTimeout(final boolean timeout) {
        this.timeout = timeout;
    }

    /**
     * @return true if method has &#64;{@link javax.interceptor.AroundInvoke}.
     */
    public boolean isAroundInvoke() {
        return this.aroundInvoke;
    }

    /**
     * Sets true if method has &#64;{@link javax.interceptor.AroundInvoke}.
     * @param aroundInvoke true/false
     */
    public void setAroundInvoke(final boolean aroundInvoke) {
        this.aroundInvoke = aroundInvoke;
    }

    /**
     * @return true if this method won't use user interceptors.
     */
    public boolean isExcludedClassInterceptors() {
        return this.excludeClassInterceptors;
    }

    /**
     * Flag this method as a method which exclude user interceptors.
     * @param excludeClassInterceptors true if this method is a method which exclude user interceptors.
     */
    public void setExcludeClassInterceptors(final boolean excludeClassInterceptors) {
        this.excludeClassInterceptors = excludeClassInterceptors;
    }

    /**
     * @return list of interceptors that enhancer will use. (ie : security/transaction)
     */
    public List<? extends IJClassInterceptor> getInterceptors() {
        return this.interceptors;
    }

    /**
     * Sets the list of interceptors(tx, security, etc) that enhancers will use.<br>
     * These interceptors are defined per methods.
     * @param interceptors list of interceptors that enhancer will use.
     */
    public void setInterceptors(final List<? extends IJClassInterceptor> interceptors) {
        this.interceptors = interceptors;
    }

    /**
     * @return object representing list of &#64;{@link javax.interceptor.Interceptors}.
     */
    public IJInterceptors getAnnotationInterceptors() {
        return this.annotationInterceptors;
    }

    /**
     * Sets the object representing the &#64;{@link javax.interceptor.Interceptors} annotation.
     * @param annotationInterceptors list of classes
     */
    public void setAnnotationsInterceptors(final IJInterceptors annotationInterceptors) {
        this.annotationInterceptors = annotationInterceptors;
    }

    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt;
     * of user interceptors that enhancer will use.
     */
    public Map<InterceptorType, List<? extends IJClassInterceptor>> getUserEasyBeansInterceptors() {
        return this.userInterceptors;
    }

    /**
     * Sets the list of user interceptors that enhancers will use.<br>
     * These interceptors are defined in bean class.
     * @param userInterceptors list of interceptors that enhancer will use.
     */
    public void setUserInterceptors(final Map<InterceptorType, List<? extends IJClassInterceptor>> userInterceptors) {
        this.userInterceptors = userInterceptors;
    }

    /**
     * @return list of global interceptors that enhancer will use. (ie : Remove interceptor)
     */
    public List<? extends IJClassInterceptor> getGlobalEasyBeansInterceptors() {
        return this.globalEasyBeansInterceptors;
    }

    /**
     * Sets the list of global interceptors that enhancers will use.
     * @param globalEasyBeansInterceptors list of interceptors that enhancer will use.
     */
    public void setGlobalEasyBeansInterceptors(final List<? extends IJClassInterceptor> globalEasyBeansInterceptors) {
        this.globalEasyBeansInterceptors = globalEasyBeansInterceptors;
    }

    /**
     * @return a clone object.
     */
    @Override
    public Object clone() {
        EjbJarMethodMetadata<E> newMethodAnnotationMetadata = new EjbJarMethodMetadata<E>(getJMethod(), getClassMetadata());
        newMethodAnnotationMetadata.setAnnotationsInterceptors(this.annotationInterceptors);
        newMethodAnnotationMetadata.setAroundInvoke(this.aroundInvoke);
        newMethodAnnotationMetadata.setBusinessMethod(this.businessMethod);
        newMethodAnnotationMetadata.setExcludeClassInterceptors(this.excludeClassInterceptors);
        newMethodAnnotationMetadata.setInterceptors(this.interceptors);
        newMethodAnnotationMetadata.setJAnnotationResource(getJAnnotationResource());
        newMethodAnnotationMetadata.setJEjbEJB(getJEjbEJB());
        newMethodAnnotationMetadata.setJavaxPersistenceContext(getJavaxPersistenceContext());
        newMethodAnnotationMetadata.setJavaxPersistenceUnit(getJavaxPersistenceUnit());
        newMethodAnnotationMetadata.setPostActivate(this.postActivate);
        newMethodAnnotationMetadata.setPostConstruct(this.postConstruct);
        newMethodAnnotationMetadata.setPreDestroy(this.preDestroy);
        newMethodAnnotationMetadata.setPrePassivate(this.prePassivate);
        newMethodAnnotationMetadata.setRemove(this.remove);
        newMethodAnnotationMetadata.setTimeout(this.timeout);
        newMethodAnnotationMetadata.setTransactionAttributeType(this.transactionAttributeType);
        newMethodAnnotationMetadata.setTransactionAttributeLevel(this.transactionAttributeLevel);
        newMethodAnnotationMetadata.setUserInterceptors(this.userInterceptors);
        newMethodAnnotationMetadata.setGlobalEasyBeansInterceptors(this.globalEasyBeansInterceptors);
        newMethodAnnotationMetadata.setDenyAll(this.denyAll);
        newMethodAnnotationMetadata.setPermitAll(this.permitAll);
        newMethodAnnotationMetadata.setRolesAllowed(this.rolesAllowed);
        return newMethodAnnotationMetadata;
    }


    /**
     * Set the list of roles allowed on this class/method.
     * @param rolesAllowed the list of roles.
     */
    public void setRolesAllowed(final List<String> rolesAllowed) {
        this.rolesAllowed = rolesAllowed;
    }

    /**
     * @return the list of roles allowed on this class/method.
     */
    public List<String> getRolesAllowed() {
        return this.rolesAllowed;
    }

    /**
     * This method has PermitAll annotation.
     * @param permitAll the boolean value.
     */
    public void setPermitAll(final boolean permitAll) {
        this.permitAll = permitAll;
    }

    /**
     * @return true if PermitAll annotation.
     */
    public boolean hasPermitAll() {
        return this.permitAll;
    }

    /**
     * This method has DenyAll annotation.
     * @param denyAll the boolean value.
     */
    public void setDenyAll(final boolean denyAll) {
        this.denyAll = denyAll;
    }

    /**
     * @return true if DenyAll annotation.
     */
    public boolean hasDenyAll() {
        return this.denyAll;
    }

    /**
     * @return true if this class won't use default interceptors.
     */
    public boolean isExcludedDefaultInterceptors() {
        return this.excludeDefaultInterceptors;
    }

    /**
     * Flag this class as a class which exclude default interceptors.
     * @param excludeDefaultInterceptors true if this class is a class which exclude default interceptors.
     */
    public void setExcludeDefaultInterceptors(final boolean excludeDefaultInterceptors) {
        this.excludeDefaultInterceptors = excludeDefaultInterceptors;
    }

}
