/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AssemblyDescriptor.java 3319 2008-05-27 13:00:31Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl.xml.struct;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.ee.metadata.ejbjar.impl.struct.JApplicationException;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.common.MethodDD;

/**
 * This class defines the implementation of the element assembly-descriptor.
 * @author Florent Benoit
 */
public class AssemblyDescriptor {

    /**
     * Name of this element.
     */
    public static final String NAME = "assembly-descriptor";

    /**
     * List of &lt;application-exception&gt; elements.
     */
    private List<JApplicationException> applicationExceptionList = null;

    /**
     * List of &lt;interceptor-binding&gt; elements.
     */
    private List<InterceptorBinding> interceptorBindingList = null;

    /**
     * List of &lt;container-transaction&gt; elements.
     */
    private List<ContainerTransaction> containerTransactionList = null;

    /**
     * List of &lt;method-permission&gt; elements.
     */
    private List<MethodPermission> methodPermissionList = null;

    /**
     * List of &lt;exclude-list&gt; elements.
     */
    private List<MethodDD> excludeListMethods = null;

    /**
     * List of &lt;security-role&gt; elements.
     */
    private List<String> securityRoleList = null;


    /**
     * Constructor.
     */
    public AssemblyDescriptor() {
        this.applicationExceptionList = new ArrayList<JApplicationException>();
        this.interceptorBindingList = new ArrayList<InterceptorBinding>();
        this.containerTransactionList = new ArrayList<ContainerTransaction>();
        this.securityRoleList = new ArrayList<String>();
        this.excludeListMethods = new ArrayList<MethodDD>();
        this.methodPermissionList = new ArrayList<MethodPermission>();
    }

    /**
     * Gets the list of &lt;application-exception&gt; elements.
     * @return list of &lt;application-exception&gt; elements.
     */
    public List<JApplicationException> getApplicationExceptionList() {
        return applicationExceptionList;
    }

    /**
     * Adds a new &lt;application-exception&gt; element to assembly descriptor.
     * @param applicationException the &lt;application-exception&gt; element
     */
    public void addApplicationException(final JApplicationException applicationException) {
        applicationExceptionList.add(applicationException);
    }

    /**
     * Gets the list of &lt;container-transaction&gt; elements.
     * @return list of &lt;container-transaction&gt; elements.
     */
    public List<ContainerTransaction> getContainerTransactionList() {
        return containerTransactionList;
    }

    /**
     * Adds a new &lt;container-transaction&gt; element to assembly descriptor.
     * @param containerTransaction the &lt;application-exception&gt; element
     */
    public void addContainerTransaction(final ContainerTransaction containerTransaction) {
        containerTransactionList.add(containerTransaction);
    }

    /**
     * Gets the list of &lt;interceptor-binding&gt; elements.
     * @return list of &lt;interceptor-binding&gt; elements.
     */
    public List<InterceptorBinding> getInterceptorBindingList() {
        return interceptorBindingList;
    }

    /**
     * Adds a new &lt;interceptor-binding&gt; element to assembly descriptor.
     * @param interceptorBinding the &lt;interceptor-binding&gt; element
     */
    public void addInterceptorBinding(final InterceptorBinding interceptorBinding) {
        interceptorBindingList.add(interceptorBinding);
    }

    /**
     * Gets the list of &lt;method-permission&gt; elements.
     * @return list of &lt;method-permission&gt; elements.
     */
    public List<MethodPermission> getMethodPermissionList() {
        return methodPermissionList;
    }

    /**
     * Adds a new &lt;method-permission&gt; element to assembly descriptor.
     * @param methodPermission the &lt;method-permission&gt; element
     */
    public void addMethodPermission(final MethodPermission methodPermission) {
        methodPermissionList.add(methodPermission);
    }

    /**
     * Gets the list of &lt;exclude-list&gt; elements.
     * @return list of &lt;exclude-list&gt; elements.
     */
    public List<MethodDD> getExcludeListMethods() {
        return excludeListMethods;
    }

    /**
     * Adds a new &lt;exclude-list&gt; element to assembly descriptor.
     * @param excludeListMethod the &lt;exclude-list&gt; element
     */
    public void addExcludeList(final MethodDD excludeListMethod) {
        excludeListMethods.add(excludeListMethod);
    }


    /**
     * Gets the list of &lt;security-role&gt; elements.
     * @return list of &lt;security-role&gt; elements.
     */
    public List<String> getSecurityRoleList() {
        return securityRoleList;
    }

    /**
     * Adds a new &lt;security-role&gt; element to assembly descriptor.
     * @param securityRole the &lt;security-role&gt; element
     */
    public void addSecurityRole(final String securityRole) {
        securityRoleList.add(securityRole);
    }
}
